@@ -1,5 +1,5 @@
+use crate::input::Input;
 use std::collections::VecDeque;
-use std::env;
 
 #[derive(Copy, Clone)]
 enum MapCell {
@@ -17,7 +17,7 @@ struct Board {
     height: u32,
     cells: Vec<MapCell>,
     visited: Vec<bool>,
-    round: i32,
+    round: u16,
     full_round: bool,
     elves_alive: i32,
     elf_died: bool,
@@ -35,6 +35,7 @@ impl Board {
         let mut goblins_alive = 0;
         let mut cells = Vec::new();
         let mut height = 0;
+
         for line in input_string.lines() {
             height += 1;
             if width as usize != line.len() {
@@ -64,7 +65,7 @@ impl Board {
             }
         }
 
-        Ok(Self {
+        let mut board = Self {
             width,
             height,
             cells,
@@ -75,7 +76,24 @@ impl Board {
             elf_died: false,
             goblins_alive,
             elf_attack_power,
-        })
+        };
+
+        for x in 0..width {
+            if !matches!(board.at(x, 0), MapCell::Wall)
+                || !matches!(board.at(x, height - 1), MapCell::Wall)
+            {
+                return Err("Map does not have a wall at edges".into());
+            }
+        }
+        for y in 0..height {
+            if !matches!(board.at(0, y), MapCell::Wall)
+                || !matches!(board.at(width - 1, y), MapCell::Wall)
+            {
+                return Err("Map does not have a wall at edges".into());
+            }
+        }
+
+        Ok(board)
     }
 
     fn at(&mut self, x: u32, y: u32) -> &mut MapCell {
@@ -99,7 +117,7 @@ impl Board {
             }
         });
         let round_for_score = self.round - if self.full_round { 0 } else { 1 };
-        Some(hit_point_sum * round_for_score)
+        Some(hit_point_sum * i32::from(round_for_score))
     }
 
     fn perform_round(&mut self) {
@@ -250,76 +268,24 @@ impl Board {
             Some((found[0].3, found[0].4))
         }
     }
-
-    fn print(&mut self) {
-        if env::var("ADVENT_DEBUG").is_err() {
-            return;
-        }
-
-        println!("Round {}", self.round);
-        for y in 0..self.height {
-            for x in 0..self.width {
-                let cell = self.at(x, y);
-                let c = match *cell {
-                    MapCell::Open => ".",
-                    MapCell::Wall => "#",
-                    MapCell::Unit { elf: false, .. } => "G",
-                    MapCell::Unit { elf: true, .. } => "E",
-                };
-                print!("{}", c);
-            }
-            print!("   ");
-            for x in 0..self.width {
-                let cell = self.at(x, y);
-                if let MapCell::Unit {
-                    hit_points,
-                    elf: true,
-                    ..
-                } = cell
-                {
-                    print!("E({}), ", hit_points);
-                } else if let MapCell::Unit {
-                    hit_points,
-                    elf: false,
-                    ..
-                } = cell
-                {
-                    print!("G({}), ", hit_points);
-                }
-            }
-            println!();
-        }
-        println!();
-    }
 }
 
-pub fn part1(input_string: &str) -> Result<i32, String> {
-    let mut board = Board::parse(input_string, 3)?;
-    board.print();
-    loop {
-        board.perform_round();
-        board.print();
-        if let Some(outcome) = board.calculate_outcome() {
-            return Ok(outcome);
-        }
-        if board.round > 500 {
-            return Err("No solution found".to_string());
-        }
-    }
-}
+pub fn solve(input: &mut Input) -> Result<i32, String> {
+    const MAX_ROUNDS: u16 = 500;
+
+    let mut attack_strength = input.part_values(3, 4);
 
-pub fn part2(input_string: &str) -> Result<i32, String> {
-    let mut attack_strength = 4;
     loop {
-        let mut board = Board::parse(input_string, attack_strength)?;
-        board.print();
+        let mut board = Board::parse(input.text, attack_strength)?;
+
         loop {
-            if board.round > 500 {
-                return Err("No solution found".to_string());
+            if board.round > MAX_ROUNDS {
+                return Err(format!("No solution found in {} rounds", MAX_ROUNDS));
             }
+
             board.perform_round();
-            board.print();
-            if board.elf_died {
+
+            if input.is_part_two() && board.elf_died {
                 break;
             } else if let Some(outcome) = board.calculate_outcome() {
                 return Ok(outcome);
@@ -331,62 +297,53 @@ pub fn part2(input_string: &str) -> Result<i32, String> {
 }
 
 #[test]
-fn tests_part1() {
-    assert_eq!(
-        Ok(27730),
-        part1(
+fn tests() {
+    use crate::{test_part_one, test_part_one_error, test_part_two};
+
+    test_part_one_error!("GE" => "No line in input");
+    test_part_one_error!("GE\n" => "Map does not have a wall at edges");
+
+    test_part_one!(
             "#######
 #.G...#
 #...EG#
 #.#.#G#
 #..G#E#
 #.....#
-#######"
-        )
+#######" => 27730
     );
 
-    assert_eq!(
-        Ok(36334),
-        part1(
+    test_part_one!(
             "#######
 #G..#E#
 #E#E.E#
 #G.##.#
 #...#E#
 #...E.#
-#######"
-        )
+#######" => 36334
     );
 
-    assert_eq!(
-        Ok(39514),
-        part1(
+    test_part_one!(
             "#######
 #E..EG#
 #.#G.E#
 #E.##E#
 #G..#.#
 #..E#.#
-#######"
-        )
+#######" => 39514
     );
 
-    assert_eq!(
-        Ok(27755),
-        part1(
+    test_part_one!(
             "#######
 #E.G#.#
 #.#G..#
 #G.#.G#
 #G..#.#
 #...E.#
-#######"
-        )
+#######"=> 27755
     );
 
-    assert_eq!(
-        Ok(28944),
-        part1(
+    test_part_one!(
             "#######
 #.E...#
 #.#..G#
@@ -394,12 +351,10 @@ fn tests_part1() {
 #E#G#G#
 #...#G#
 #######"
-        )
+        =>28944
     );
 
-    assert_eq!(
-        Ok(18740),
-        part1(
+    test_part_one!(
             "#########
 #G......#
 #.E.#...#
@@ -409,39 +364,31 @@ fn tests_part1() {
 #.G...G.#
 #.....G.#
 #########"
-        )
+        =>18740
     );
 
-    assert_eq!(Ok(207_059), part1(include_str!("day15_input.txt")));
-}
+    let input = include_str!("day15_input.txt");
+    test_part_one!(input => 207_059);
 
-#[test]
-fn tests_part2() {
-    assert_eq!(
-        Ok(4988),
-        part2(
+    test_part_two!(
             "#######
 #.G...#
 #...EG#
 #.#.#G#
 #..G#E#
 #.....#
-#######"
-        )
+#######" =>4988
     );
 
-    assert_eq!(
-        Ok(31284),
-        part2(
+    test_part_two!(
             "#######
 #E..EG#
 #.#G.E#
 #E.##E#
 #G..#.#
 #..E#.#
-#######"
-        )
+#######" =>31284
     );
 
-    assert_eq!(Ok(49120), part2(include_str!("day15_input.txt")));
+    test_part_two!(input => 49120);
 }
